"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const pm2 = require("pm2");
const handlers = {};
const myPmId = Number(process.env.pm_id);
const myName = process.env.name;
process.on('message', ({ topic, data: { targetInstanceId, data } }) => __awaiter(this, void 0, void 0, function* () {
    if (typeof handlers[topic] === 'function' && process.send) {
        const response = {
            type: `process:${targetInstanceId}`,
            data: yield handlers[topic](data),
        };
        process.send(response);
    }
}));
/**
 * Requests messages from processes managed by pm2.
 */
exports.getMessages = function getMessages(topic, data, { filter = (process) => process.name === myName, includeSelfIfUnmanaged = false, timeout = 1000, } = {}) {
    return new Promise((resolve, reject) => {
        const timer = setTimeout(() => reject(new Error(`${topic} timed out`)), timeout);
        const done = function done(err, messages) {
            clearTimeout(timer);
            if (err)
                reject(err);
            else
                resolve(messages);
        };
        pm2.connect(false, (err) => {
            if (err)
                return done(err, []);
            new Promise((resolve, reject) => {
                pm2.list((err, processes) => {
                    if (err)
                        return reject(err);
                    const messages = [];
                    const resolvers = [];
                    const resolverBusTargets = [];
                    const resolverSelf = () => __awaiter(this, void 0, void 0, function* () { messages.push(yield handlers[topic](data)); });
                    if (includeSelfIfUnmanaged && Number.isNaN(myPmId))
                        resolvers.push(resolverSelf());
                    for (const process of processes) {
                        if (filter(process)) {
                            if (process.pm_id === myPmId) {
                                resolvers.push(resolverSelf());
                            }
                            else if (typeof process.pm_id === 'number') {
                                resolverBusTargets.push(process.pm_id);
                            }
                        }
                    }
                    console.info(resolvers.length, resolverBusTargets.length);
                    if (resolverBusTargets.length) {
                        resolvers.push(new Promise((resolve, reject) => {
                            pm2.launchBus((err, bus) => {
                                if (err)
                                    return reject(err);
                                let pending = resolverBusTargets.length;
                                bus.on(`process:${myPmId}`, ({ data }) => {
                                    messages.push(data);
                                    pending -= 1;
                                    if (!pending)
                                        resolve();
                                });
                                const request = { topic, data: { targetInstanceId: myPmId, data } };
                                resolverBusTargets.forEach((pmId) => {
                                    pm2.sendDataToProcessId(pmId, request, (err) => err && reject(err));
                                });
                            });
                        }));
                    }
                    Promise.all(resolvers)
                        .then(() => resolve(messages))
                        .catch(reject);
                });
            })
                .then((messages) => done(null, messages))
                .catch((err) => done(err, []))
                .finally(() => pm2.disconnect());
        });
    });
};
/**
 * Attaches a handler for the given topic.
 */
exports.onMessage = function onMessage(topic, handler) {
    handlers[topic] = handler;
};
exports.default = { getMessages: exports.getMessages, onMessage: exports.onMessage };
//# sourceMappingURL=index.js.map